{%- extends "alabaster/layout.html" %}

{%- block footer %}
  <div class="footer">
    {% if show_copyright %}&copy;{{ copyright }} | {% endif %}
    Licensed under the <a href="https://creativecommons.org/licenses/by-sa/4.0/">
    CC BY-SA 4.0</a> license
    {%- if show_source and has_source and sourcename %}
      {% if show_copyright or theme_show_powered_by %}|{% endif %}
      <a href="{{ pathto('_sources/' + sourcename, true)|e }}"
         rel="nofollow">{{ _('Page source') }}</a>
    {%- endif %}
  </div>

{% endblock %}
