{% extends "app.html" %}
{% comment %}
# SPDX-License-Identifier: AGPL-3.0-or-later
{% endcomment %}

{% load bootstrap %}
{% load i18n %}

{% block configuration %}
  {{ block.super }}

  <h3>{% trans "I2P Proxies and Tunnels" %}</h3>
  {% for line in proxies_description %}
    <p>{{ line|safe }}</p>
  {% endfor %}

  <p>
    <a class="btn btn-primary" target="_blank" role="button"
       href="/i2p/i2ptunnel/"
       {{ is_enabled|yesno:',disabled="disabled"' }}>
      {% trans "Launch" %}
    </a>
  </p>

  <h3>{% trans "Anonymous Torrents" %}</h3>
  {% for line in torrents_description %}
    <p>{{ line|safe }}</p>
  {% endfor %}

  <p>
    <a class="btn btn-primary" target="_blank" role="button"
       href="/i2p/i2psnark/"
       {{ is_enabled|yesno:',disabled="disabled"' }}>
      {% trans "Launch" %}
    </a>
  </p>
{% endblock %}
