{% comment %}
# SPDX-License-Identifier: AGPL-3.0-or-later
{% endcomment %}

{% load i18n %}

<div class="table-responsive">
  <table class="table diagnostics-results">
    <thead>
      <tr>
        <th class="diagnostic-test">{% trans "Test" %}</th>
        <th class="diagnostics-result">{% trans "Result" %}</th>
      </tr>
    </thead>
    <tbody>
      {% for result in results %}
        <tr>
          <td>{{ result.translated_description }}</td>
          <td class="diagnostics-result">
            {% if result.result == 'passed' %}
              <span class="badge badge-success">{% trans result.result %}</span>
            {% elif result.result == 'failed' %}
              <span class="badge badge-danger">{% trans result.result %}</span>
            {% elif result.result == 'error' or result.result == 'warning' %}
              <span class="badge badge-warning">{% trans result.result %}</span>
            {% else %}
              {{ result.result }}
            {% endif %}
          </td>
        </tr>
      {% endfor %}
    </tbody>
  </table>
</div>
