{% extends "base.html" %}
{% comment %}
# SPDX-License-Identifier: AGPL-3.0-or-later
{% endcomment %}

{% load i18n %}

{% block content %}

  {% for peer in server.peers.values %}
    {% if forloop.first %}
      <h3>{{ title }}</h3>
      <p>
        {% blocktrans trimmed %}
          {{ box_name }} will attempt to reach a WireGuard server with the
          following information. Ensure that the server is configured to allow
          {{ box_name }}'s public key and IP address.
        {% endblocktrans %}
      </p>
      <div class="table-responsive">
        <table class="table">
          <tbody>
            <tr class="peer-endpoint">
              <th>{% trans "Server endpoint:" %}</th>
              <td>{{ peer.endpoint }}</td>
            </tr>
            <tr class="peer-public-key">
              <th>{% trans "Server public key:" %}</th>
              <td>{{ peer.public_key }}</td>
            </tr>
            <tr class="peer-preshared-key">
              <th>{% trans "Pre-shared key:" %}</th>
              <td>{{ peer.preshared_key }}</td>
            </tr>
            <tr class="server-public-key">
              <th>{% trans "Public key of this machine:" %}</th>
              <td>{{ server.public_key }}</td>
            </tr>
            <tr class="server-ip-address">
              <th>{% trans "IP address of this machine:" %}</th>
              <td>{{ server.ip_address }}</td>
            </tr>
          </tbody>
        </table>
      </div>

      <h3>{% trans "Status" %}</h3>
      <div class="table-responsive">
        <table class="table">
          <tbody>
            <tr class="peer-transfer-tx">
              <th>{% trans "Data transmitted:" %}</th>
              <td>{{ peer.status.transfer_tx|filesizeformat }}</td>
            </tr>
            <tr class="peer-transfer-rx">
              <th>{% trans "Data received:" %}</th>
              <td>{{ peer.status.transfer_rx|filesizeformat }}</td>
            </tr>
            <tr class="peer-latest-handshake">
              <th>{% trans "Latest handshake:" %}</th>
              <td>{{ peer.status.latest_handshake|default:'' }}</td>
            </tr>
          </tbody>
        </table>
      </div>
    {% endif %}
  {% endfor %}

  <div class="btn-toolbar">
    <a class="btn btn-default btn-edit-server"
       href="{% url 'wireguard:edit-server' interface %}">
      <span class="fa fa-pencil-square-o" aria-hidden="true"></span>
      {% trans "Edit" %}
    </a>
    <a class="btn btn-default btn-delete-server"
       href="{% url 'wireguard:delete-server' interface %}">
      <span class="fa fa-trash-o" aria-hidden="true"></span>
      {% trans "Delete" %}
    </a>
  </div>

{% endblock %}
