{% extends "base.html" %}
{% comment %}
# SPDX-License-Identifier: AGPL-3.0-or-later
{% endcomment %}

{% load bootstrap %}
{% load i18n %}

{% block content %}
  <h3>
    {% blocktrans trimmed with username=form.user.username %}
      Change Password for <em>{{ username }}</em>
    {% endblocktrans %}
  </h3>
  <form class="form form-change-password" method="post">
    {% csrf_token %}

    {{ form|bootstrap }}

    <input type="submit" class="btn btn-primary"
           value="{% trans "Save Password" %}"/>
  </form>
{% endblock %}

