{% comment %}
# SPDX-License-Identifier: AGPL-3.0-or-later
{% endcomment %}

{# Template to display notifications under the navbar #}

{% load i18n %}
{% load static %}

{% if notifications %}
  <div class="notifications dropdown">
    <ul class="dropdown-menu" role="menu">
      {% for note in notifications %}
        <li class="notification notification-{{ note.severity }}">
          {% if note.data.app_name %}
            <div class="app-name">
              {% if note.data.app_icon %}
                <div class="app-icon fa {{ note.data.app_icon }}"></div>
              {% elif note.data.app_icon_filename %}
                {% if note.app_id %}
                  <img src="{% static note.app_id %}/icons/{{ note.data.app_icon_filename }}.svg"
                       alt="{{ note.data.app_name }}"
                       class="notification-icon" />
                {% else %}
                  <img src="{% static 'theme/icons/' %}{{ note.data.app_icon_filename }}.svg"
                       alt="{{ note.data.app_name }}"
                       class="notification-icon" />
                {% endif %}
              {% endif %}

              {{ note.data.app_name }}
            </div>
          {% endif %}

          {% if note.body %}
            {{ note.body.content.decode|safe }}
          {% else %}
            <div class="notification-title">{{ note.title }}</div>

            {% if note.message %}
              <p>{{ note.message }}</p>
            {% endif %}

            {% if note.actions %}
              <div class="btn-toolbar">
                {% for action in note.actions %}
                  {% if action.type == "dismiss" %}
                    <a href="{% url 'notification_dismiss' id=note.id %}?next={{ request.path|iriencode }}"
                       role="button" class="btn btn-default">
                      {% trans "Dismiss" %}
                    </a>
                  {% else %}
                    <a href="{% url action.url %}" role="button"
                       class="btn btn-{{ action.class|default:'default' }}">
                      {{ action.text }}
                    </a>
                  {% endif %}
                {% endfor %}
              </div>
            {% endif %}
          {% endif %}
        </li>
      {% endfor %}
    </ul>
  </div>
{% endif %}
