/*
# SPDX-License-Identifier: AGPL-3.0-or-later
*/

a.dropdown-toggle {
    color: black;
}

a.dropdown-toggle:hover {
    text-decoration: none;
}

td.service {
    padding-left: 2rem;
}

.service-name {
    font-style: italic;
}

.app-status,
.service-status {
    width: 11rem;
}

tr.collapse {
    background-color: #f9f9f9;
}
