{% extends "base.html" %}
{% comment %}
# SPDX-License-Identifier: AGPL-3.0-or-later
{% endcomment %}

{% load bootstrap %}
{% load i18n %}

{% block content %}

  {{ block.super }}

  <h2>{% trans 'Manage Aliases' %}</h2>

  {% if not list_form.fields.aliases.choices %}
    <p>{% trans "You have no email aliases." %}</p>
  {% else %}
    <form action="{{ request.path }}" method="post">
      {% csrf_token %}
      {{ list_form|bootstrap }}

      <input type="hidden" name="form" value="list">
      <input class="btn btn-danger" type="submit" name="btn_delete"
             value="{% trans 'Delete' %}">
    </form>
  {% endif %}

  <h4>{% trans "Create a new email alias" %}</h4>

  <form action="{{ request.path }}" method="post">
    {% csrf_token %}
    {{ create_form|bootstrap }}
    <input type="hidden" name="form" value="create">
    <input class="btn btn-primary" type="submit" value="{% trans 'Add' %}">
  </form>

{% endblock %}
