{% comment %}
# SPDX-License-Identifier: AGPL-3.0-or-later
{% endcomment %}

{% load bootstrap %}
{% load i18n %}
{% load static %}

<h1>
  {% blocktrans trimmed %}
    What Type Of Internet Connection Do You Have?
  {% endblocktrans %}
</h1>

<p>
  {% blocktrans trimmed %}
    Select an option that best describes the type of Internet connection.
    This information is used only to guide you with further setup.
  {% endblocktrans %}
</p>
