/*
# SPDX-License-Identifier: AGPL-3.0-or-later
*/

.connection-status-cell {
    width: 4.125rem;
}

.connection-status-label {
    width: 4.125rem;
    padding: 0.3125rem 0;
    text-align: center;
    margin-right: 0.625rem;
}

.connection-actions {
    width: 10rem;
}

.form-action {
    display: inline;
}

.form-action button {
    min-width: 6.25rem;
}


/* Connection diagram */
.connection-diagram {
    margin-top: 2rem;
    margin-bottom: 2rem;
}

.col-image {
    display: block;
    width: 100%;
}

.network-entity {
    max-width: 8rem;
}

.network-type-icon {
    width: 1.5rem;
    height: 1.5rem;
}

.network-connection {
    height: 0.5rem;
    margin-bottom: 0.75rem;
}

.network-connection-vertical {
    width: 0.5rem;
    margin-left: 3.75rem;
}

.connection-list {
    margin-left: 1.5rem;
}

.connection-list-heading {
    font-weight: bold;
}
