# SPDX-License-Identifier: AGPL-3.0-or-later

backup = {
    'config': {
        'directories': ['/etc/ez-ipupdate/']
    },
    'settings': [
        'dynamicdns_enable', 'dynamicdns_config', 'dynamicdns_status'
    ],
}
