{% extends "app.html" %}
{% comment %}
# SPDX-License-Identifier: AGPL-3.0-or-later
{% endcomment %}
{% load bootstrap %}
{% load i18n %}

{% block status %}
  {{ block.super }}

  <h3>{% trans "Serving Domains" %}</h3>

  <div class="table-responsive">
    <table class="table">
      <thead>
        <tr>
          <th>{% trans "Type" %}</th>
          <th>{% trans "Domain Names" %}</th>
          <th>{% trans "Serving" %}</th>
          <th>{% trans "IP addresses" %}</th>
        </tr>
      </thead>
      <tbody>
        {% for domain in domains_table %}
          <tr>
            <td>{{ domain.type}}</td>
            <td>{{ domain.domain_name}}</td>
            <td>{{ domain.serving }}</td>
            <td>{{ domain.ip_address }}</td>
          </tr>
        {% endfor %}
      </tbody>
    </table>
  </div>
  <div class="btn-toolbar">
    <a href="{% url 'bind:index' %}" class="btn btn-default"
       role="button" title="{% trans 'Refresh IP address and domains' %}">
      <span class="fa" aria-hidden="true"></span>
        {% trans 'Refresh IP address and domains' %}
    </a>
  </div>
{% endblock %}
