{% extends 'base.html' %}
{% comment %}
# SPDX-License-Identifier: AGPL-3.0-or-later
{% endcomment %}

{% load i18n %}

{% block content %}

  <h2>{% trans "Diagnostic Results" %}</h2>

  <div class="row align-items-center justify-content-between">
    <h3>{% blocktrans %}App: {{ app_name }}{% endblocktrans %}</h3>

    {% if show_repair %}
      <form class="form form-diagnostics-repair-button" method="post"
            action="{% url 'diagnostics:repair' app_id=app_id %}">
        {% csrf_token %}
        <input type="submit" class="btn btn-default"
               name="repair" value="{% trans "Try to repair" %}"/>
      </form>
    {% endif %}
  </div>

  {% if results %}
    {% include "diagnostics_results.html" with results=results %}
  {% elif exception %}
    <div class="alert alert-danger" role="alert">
      {{ exception }}
    </div>
  {% else %}
    <p>{% trans "This app does not support diagnostics" %}</p>
  {% endif %}

{% endblock %}
