{% comment %}
# SPDX-License-Identifier: AGPL-3.0-or-later
{% endcomment %}

{% load i18n %}

{% for message in messages %}
  <div class='alert alert-{{ message.tags }} alert-dismissible'>
    {{ message }}
    <button type="button" class="close" data-dismiss="alert"
            aria-label="{% trans "Close" %}">
      <span aria-hidden="true">&times;</span>
    </button>
  </div>
{% endfor %}
