{% extends "app.html" %}
{% comment %}
# SPDX-License-Identifier: AGPL-3.0-or-later
{% endcomment %}

{% load bootstrap %}
{% load i18n %}

{% block status %}
  {{ block.super }}

  <h3>Status</h3>

  <p>
    {% blocktrans trimmed %}
      Use the following URLs to configure your communication server:
    {% endblocktrans %}

    <pre>{% for uri in config.uris %}{{ uri }}
{% endfor %}</pre>
  </p>

  <p>
    {% blocktrans trimmed %}
      Use the following shared authentication secret:
    {% endblocktrans %}

    <pre>{{ config.shared_secret }}</pre>
  </p>
{% endblock %}
